# frozen_string_literal: true

RSpec.describe 'simple_form_for with a symbolic name as a passed object' do
  context 'when the `element_name_separator` is set to `___`' do
    before do
      Bemer.config.element_name_separator = '___'
    end

    it do
      form = simple_form_for :block do |f|
        f.input :elem
      end

      expect(form).to have_tag(:form, with: { class: 'block' }, count: 1) do
        with_tag :input, with: { class: 'block___elem' }, count: 1
      end
    end
  end

  context 'when the `modifier_name_separator` is set to `--`' do
    before do
      Bemer.config.modifier_name_separator = '--'
    end

    it do
      form = simple_form_for :block, mods: :enabled do |f|
        f.input :elem
      end

      expect(form).to have_tag(:form, with: { class: 'block block--enabled' }, count: 1) do
        with_tag :input, with: { class: 'block__elem block__elem--string' }, count: 1
      end
    end
  end

  context 'when the `modifier_value_separator` is set to `___`' do
    before do
      Bemer.config.modifier_value_separator = '___'
    end

    it do
      form = simple_form_for :block, mods: { size: :large } do |f|
        f.input :elem, input_html: { mods: { disabled: :yes } }
      end

      expect(form).to have_tag(:form, with: { class: 'block block_size___large' }, count: 1) do
        with_tag :input, with: { class: 'block__elem block__elem_disabled___yes' }, count: 1
      end
    end
  end

  context 'when the `bem` is globally set to `false`' do
    before do
      Bemer.config.bem = false
    end

    it do
      form = simple_form_for :block do |f|
        f.input :elem
      end

      expect(form).to have_tag(:form, count: 1) do
        without_tag 'form[class]'
        without_tag 'input[class]'
      end
    end
  end

  it do
    form = simple_form_for Article.new, as: 'sss_sss' do |f|
      # f.input 'elem_name'
      f.input :title
    end

    form = simple_form_for 'eferg_adFdb', as: 'sss_sss' do |f|
      f.input 'elem_name'
      # f.input :title
    end

    p form
    # expect(form).to have_tag(:form, count: 1) do
    #   without_tag 'form[class]'
    #   without_tag 'input[class]'
    # end
  end
end
